-----------------------------------------------------------------------------------------------
-- Client Lua Script for SAmotes
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- SAmotes Module Definition
-----------------------------------------------------------------------------------------------
local SAmotes = {} 


-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------

local SAmotesSpecial = {} -- for handling of emotes not bracketed with colons
SAmotesSpecial[":%)"] = ":):" -- brackets are escaped with % so they don't mess with string.find()
SAmotesSpecial[":%("] = ":(:"
SAmotesSpecial[":D"] = ":dbiggrin:" -- sprite names are not case-sensitive so name change needed to avoid clashing with :d:
SAmotesSpecial[";%)"] = ":;):"
SAmotesSpecial[";%-%*"] = ":;-*:"

local SAmotesData = {}
SAmotesData[":(:"] = { w = 15, h = 15 }
SAmotesData[":):"] = { w = 15, h = 15 }
SAmotesData[":10bux:"] = { w = 30, h = 15 }
SAmotesData[":11tea:"] = { w = 34, h = 27 }
SAmotesData[":20bux:"] = { w = 30, h = 15 }
SAmotesData[":2bong:"] = { w = 42, h = 19 }
SAmotesData[":350:"] = { w = 25, h = 27 }
SAmotesData[":3:"] = { w = 17, h = 20 }
SAmotesData[":420:"] = { w = 15, h = 14 }
SAmotesData[":69snypa:"] = { w = 84, h = 27 }
SAmotesData[":911:"] = { w = 33, h = 17 }
SAmotesData[":?:"] = { w = 18, h = 18 }
SAmotesData[":FYH:"] = { w = 102, h = 27 }
SAmotesData[":a2m:"] = { w = 42, h = 21 }
SAmotesData[":aaa:"] = { w = 17, h = 17 }
SAmotesData[":aaaaa:"] = { w = 17, h = 17 }
SAmotesData[":aatrek:"] = { w = 50, h = 25 }
SAmotesData[":agesilaus:"] = { w = 26, h = 25 }
SAmotesData[":airquote:"] = { w = 37, h = 19 }
SAmotesData[":allears:"] = { w = 25, h = 20 }
SAmotesData[":am:"] = { w = 16, h = 16 }
SAmotesData[":angel:"] = { w = 44, h = 25 }
SAmotesData[":anime:"] = { w = 24, h = 25 }
SAmotesData[":argh:"] = { w = 34, h = 17 }
SAmotesData[":arghfist:"] = { w = 14, h = 17 }
SAmotesData[":aslol:"] = { w = 24, h = 27 }
SAmotesData[":asoiaf:"] = { w = 92, h = 22 }
SAmotesData[":australia:"] = { w = 34, h = 18 }
SAmotesData[":awesome:"] = { w = 40, h = 20 }
SAmotesData[":awesomelon:"] = { w = 22, h = 22 }
SAmotesData[":axe:"] = { w = 27, h = 28 }
SAmotesData[":baby:"] = { w = 27, h = 25 }
SAmotesData[":backtowork:"] = { w = 38, h = 25 }
SAmotesData[":bahgawd:"] = { w = 42, h = 27 }
SAmotesData[":bandwagon:"] = { w = 102, h = 27 }
SAmotesData[":bang:"] = { w = 27, h = 22 }
SAmotesData[":banjo:"] = { w = 21, h = 22 }
SAmotesData[":banme:"] = { w = 60, h = 15 }
SAmotesData[":barf:"] = { w = 62, h = 27 }
SAmotesData[":beck:"] = { w = 52, h = 27 }
SAmotesData[":belarus:"] = { w = 33, h = 17 }
SAmotesData[":bick:"] = { w = 32, h = 27 }
SAmotesData[":bigtran:"] = { w = 19, h = 19 }
SAmotesData[":biotruths:"] = { w = 82, h = 20 }
SAmotesData[":black101:"] = { w = 35, h = 22 }
SAmotesData[":blush:"] = { w = 17, h = 17 }
SAmotesData[":boonie:"] = { w = 44, h = 27 }
SAmotesData[":bravo2:"] = { w = 52, h = 20 }
SAmotesData[":bravo:"] = { w = 98, h = 27 }
SAmotesData[":britain:"] = { w = 33, h = 18 }
SAmotesData[":bsg:"] = { w = 68, h = 27 }
SAmotesData[":btroll:"] = { w = 50, h = 32 }
SAmotesData[":bubblewoop:"] = { w = 22, h = 27 }
SAmotesData[":buddy:"] = { w = 19, h = 20 }
SAmotesData[":bunt:"] = { w = 92, h = 27 }
SAmotesData[":burger:"] = { w = 27, h = 27 }
SAmotesData[":bustem:"] = { w = 100, h = 25 }
SAmotesData[":butt:"] = { w = 15, h = 15 }
SAmotesData[":byewhore:"] = { w = 60, h = 17 }
SAmotesData[":byob1:"] = { w = 26, h = 25 }
SAmotesData[":byob:"] = { w = 94, h = 22 }
SAmotesData[":byobear:"] = { w = 67, h = 17 }
SAmotesData[":byodame:"] = { w = 19, h = 19 }
SAmotesData[":byodood:"] = { w = 19, h = 19 }
SAmotesData[":c00l:"] = { w = 17, h = 17 }
SAmotesData[":c00lbert:"] = { w = 26, h = 26 }
SAmotesData[":c00lbutt:"] = { w = 26, h = 26 }
SAmotesData[":c:"] = { w = 13, h = 11 }
SAmotesData[":ca:"] = { w = 42, h = 27 }
SAmotesData[":cabot:"] = { w = 29, h = 25 }
SAmotesData[":cadfan:"] = { w = 87, h = 26 }
SAmotesData[":camera6:"] = { w = 49, h = 25 }
SAmotesData[":can:"] = { w = 27, h = 27 }
SAmotesData[":canada:"] = { w = 33, h = 17 }
SAmotesData[":catbert:"] = { w = 21, h = 23 }
SAmotesData[":catdrugs:"] = { w = 102, h = 27 }
SAmotesData[":catholic:"] = { w = 27, h = 25 }
SAmotesData[":catstare:"] = { w = 24, h = 18 }
SAmotesData[":cawg:"] = { w = 25, h = 27 }
SAmotesData[":cb:"] = { w = 17, h = 17 }
SAmotesData[":ccb:"] = { w = 102, h = 27 }
SAmotesData[":cedric:"] = { w = 39, h = 25 }
SAmotesData[":cenobite:"] = { w = 15, h = 15 }
SAmotesData[":cheat:"] = { w = 33, h = 17 }
SAmotesData[":cheeky:"] = { w = 15, h = 15 }
SAmotesData[":cheers:"] = { w = 29, h = 15 }
SAmotesData[":chef:"] = { w = 25, h = 25 }
SAmotesData[":chiefsay:"] = { w = 36, h = 27 }
SAmotesData[":chillpill:"] = { w = 27, h = 15 }
SAmotesData[":china:"] = { w = 33, h = 17 }
SAmotesData[":chiyo:"] = { w = 122, h = 17 }
SAmotesData[":choco:"] = { w = 54, h = 19 }
SAmotesData[":chord:"] = { w = 53, h = 24 }
SAmotesData[":circlefap:"] = { w = 45, h = 21 }
SAmotesData[":clegg:"] = { w = 48, h = 27 }
SAmotesData[":clint:"] = { w = 32, h = 27 }
SAmotesData[":cmon:"] = { w = 74, h = 25 }
SAmotesData[":coal:"] = { w = 15, h = 15 }
SAmotesData[":coffee:"] = { w = 14, h = 19 }
SAmotesData[":colbert:"] = { w = 21, h = 23 }
SAmotesData[":comeback:"] = { w = 19, h = 19 }
SAmotesData[":commissar:"] = { w = 66, h = 26 }
SAmotesData[":confuoot:"] = { w = 29, h = 25 }
SAmotesData[":confused:"] = { w = 15, h = 22 }
SAmotesData[":cool:"] = { w = 15, h = 15 }
SAmotesData[":coolfish:"] = { w = 57, h = 21 }
SAmotesData[":cop:"] = { w = 28, h = 21 }
SAmotesData[":corrupt:"] = { w = 92, h = 27 }
SAmotesData[":corsair:"] = { w = 29, h = 31 }
SAmotesData[":coupons:"] = { w = 99, h = 22 }
SAmotesData[":cripes:"] = { w = 21, h = 23 }
SAmotesData[":crossarms:"] = { w = 32, h = 17 }
SAmotesData[":crow:"] = { w = 28, h = 21 }
SAmotesData[":cry:"] = { w = 18, h = 18 }
SAmotesData[":cthulhu:"] = { w = 27, h = 25 }
SAmotesData[":cult:"] = { w = 23, h = 24 }
SAmotesData[":d:"] = { w = 13, h = 11 }
SAmotesData[":dbiggrin:"] = { w = 16, h = 16 }
SAmotesData[":damn:"] = { w = 65, h = 27 }
SAmotesData[":dance:"] = { w = 26, h = 20 }
SAmotesData[":darksouls:"] = { w = 92, h = 27 }
SAmotesData[":dawkins101:"] = { w = 46, h = 27 }
SAmotesData[":dealwithit:"] = { w = 52, h = 24 }
SAmotesData[":denmark:"] = { w = 33, h = 17 }
SAmotesData[":derp:"] = { w = 52, h = 27 }
SAmotesData[":derptiel:"] = { w = 15, h = 25 }
SAmotesData[":devil:"] = { w = 24, h = 23 }
SAmotesData[":devilchild:"] = { w = 63, h = 27 }
SAmotesData[":dice:"] = { w = 56, h = 17 }
SAmotesData[":dings:"] = { w = 102, h = 27 }
SAmotesData[":dogout:"] = { w = 102, h = 27 }
SAmotesData[":doh:"] = { w = 21, h = 22 }
SAmotesData[":doink:"] = { w = 41, h = 25 }
SAmotesData[":dominic:"] = { w = 17, h = 17 }
SAmotesData[":dong:"] = { w = 32, h = 17 }
SAmotesData[":doom:"] = { w = 33, h = 21 }
SAmotesData[":dota101:"] = { w = 42, h = 25 }
SAmotesData[":douche:"] = { w = 42, h = 27 }
SAmotesData[":downs:"] = { w = 15, h = 15 }
SAmotesData[":downsbravo:"] = { w = 54, h = 27 }
SAmotesData[":downsgun:"] = { w = 29, h = 21 }
SAmotesData[":downsowned:"] = { w = 97, h = 19 }
SAmotesData[":downsrim:"] = { w = 39, h = 27 }
SAmotesData[":downswords:"] = { w = 48, h = 17 }
SAmotesData[":drac:"] = { w = 31, h = 25 }
SAmotesData[":drugnerd:"] = { w = 33, h = 21 }
SAmotesData[":drum:"] = { w = 28, h = 25 }
SAmotesData[":ducksiren:"] = { w = 48, h = 27 }
SAmotesData[":dukedog:"] = { w = 58, h = 25 }
SAmotesData[":edi:"] = { w = 65, h = 22 }
SAmotesData[":eek:"] = { w = 15, h = 15 }
SAmotesData[":effort:"] = { w = 86, h = 19 }
SAmotesData[":emo:"] = { w = 38, h = 17 }
SAmotesData[":emoticon:"] = { w = 102, h = 27 }
SAmotesData[":eng101:"] = { w = 23, h = 19 }
SAmotesData[":eng99:"] = { w = 23, h = 19 }
SAmotesData[":engleft:"] = { w = 25, h = 21 }
SAmotesData[":ese:"] = { w = 32, h = 27 }
SAmotesData[":eurovision:"] = { w = 32, h = 16 }
SAmotesData[":evil:"] = { w = 41, h = 18 }
SAmotesData[":eyepop:"] = { w = 22, h = 21 }
SAmotesData[":f5:"] = { w = 30, h = 19 }
SAmotesData[":f5h:"] = { w = 15, h = 19 }
SAmotesData[":faggot:"] = { w = 30, h = 25 }
SAmotesData[":fap:"] = { w = 43, h = 21 }
SAmotesData[":feelsgood:"] = { w = 87, h = 19 }
SAmotesData[":fella:"] = { w = 24, h = 25 }
SAmotesData[":fh:"] = { w = 27, h = 21 }
SAmotesData[":fiesta:"] = { w = 41, h = 24 }
SAmotesData[":filez:"] = { w = 102, h = 27 }
SAmotesData[":fireman:"] = { w = 102, h = 27 }
SAmotesData[":firstpost:"] = { w = 45, h = 15 }
SAmotesData[":flaccid:"] = { w = 32, h = 27 }
SAmotesData[":flag:"] = { w = 22, h = 25 }
SAmotesData[":flame:"] = { w = 62, h = 27 }
SAmotesData[":flashfact:"] = { w = 36, h = 22 }
SAmotesData[":flashfap:"] = { w = 46, h = 21 }
SAmotesData[":fork:"] = { w = 30, h = 25 }
SAmotesData[":forkbomb:"] = { w = 92, h = 27 }
SAmotesData[":foxnews:"] = { w = 28, h = 25 }
SAmotesData[":france:"] = { w = 28, h = 17 }
SAmotesData[":freep:"] = { w = 62, h = 27 }
SAmotesData[":frog:"] = { w = 38, h = 25 }
SAmotesData[":frogbon:"] = { w = 38, h = 25 }
SAmotesData[":frogc00l:"] = { w = 40, h = 27 }
SAmotesData[":frogdowns:"] = { w = 38, h = 25 }
SAmotesData[":froggonk:"] = { w = 38, h = 25 }
SAmotesData[":frogon:"] = { w = 73, h = 25 }
SAmotesData[":frogout:"] = { w = 102, h = 27 }
SAmotesData[":frogsiren:"] = { w = 40, h = 27 }
SAmotesData[":fry:"] = { w = 66, h = 24 }
SAmotesData[":fsmug:"] = { w = 43, h = 25 }
SAmotesData[":fsn:"] = { w = 35, h = 22 }
SAmotesData[":ftbrg:"] = { w = 19, h = 19 }
SAmotesData[":fuckoff:"] = { w = 62, h = 27 }
SAmotesData[":furcry:"] = { w = 33, h = 22 }
SAmotesData[":fut:"] = { w = 58, h = 18 }
SAmotesData[":fyadride:"] = { w = 31, h = 25 }
SAmotesData[":gaben:"] = { w = 33, h = 17 }
SAmotesData[":gay:"] = { w = 35, h = 19 }
SAmotesData[":gb2byob:"] = { w = 52, h = 22 }
SAmotesData[":gb2fyad:"] = { w = 52, h = 22 }
SAmotesData[":gb2gbs:"] = { w = 31, h = 17 }
SAmotesData[":gb2hd2k:"] = { w = 100, h = 25 }
SAmotesData[":gbsmith:"] = { w = 25, h = 27 }
SAmotesData[":geert:"] = { w = 33, h = 23 }
SAmotesData[":geno:"] = { w = 15, h = 15 }
SAmotesData[":george:"] = { w = 49, h = 32 }
SAmotesData[":gerty:"] = { w = 25, h = 25 }
SAmotesData[":getin:"] = { w = 64, h = 27 }
SAmotesData[":getout:"] = { w = 30, h = 25 }
SAmotesData[":ghost:"] = { w = 17, h = 24 }
SAmotesData[":gibs:"] = { w = 15, h = 15 }
SAmotesData[":gifttank:"] = { w = 60, h = 27 }
SAmotesData[":gizz:"] = { w = 57, h = 27 }
SAmotesData[":glomp:"] = { w = 49, h = 22 }
SAmotesData[":goatsecx:"] = { w = 29, h = 17 }
SAmotesData[":godwin:"] = { w = 99, h = 22 }
SAmotesData[":godwinning:"] = { w = 37, h = 27 }
SAmotesData[":goleft:"] = { w = 15, h = 15 }
SAmotesData[":golfclap:"] = { w = 26, h = 17 }
SAmotesData[":golgo:"] = { w = 74, h = 27 }
SAmotesData[":gonchar:"] = { w = 25, h = 27 }
SAmotesData[":gonk:"] = { w = 15, h = 15 }
SAmotesData[":goof:"] = { w = 35, h = 16 }
SAmotesData[":google:"] = { w = 56, h = 20 }
SAmotesData[":goon:"] = { w = 62, h = 27 }
SAmotesData[":goonboot:"] = { w = 22, h = 27 }
SAmotesData[":gooncamp:"] = { w = 90, h = 41 }
SAmotesData[":goonsay:"] = { w = 86, h = 25 }
SAmotesData[":grandma:"] = { w = 15, h = 15 }
SAmotesData[":greencube:"] = { w = 43, h = 46 }
SAmotesData[":greatgift:"] = { w = 16, h = 16 }
SAmotesData[":gtfoycs:"] = { w = 35, h = 17 }
SAmotesData[":guinness:"] = { w = 47, h = 22 }
SAmotesData[":guitar:"] = { w = 29, h = 21 }
SAmotesData[":gurf:"] = { w = 27, h = 21 }
SAmotesData[":h:"] = { w = 13, h = 11 }
SAmotesData[":hampants:"] = { w = 36, h = 22 }
SAmotesData[":happyelf:"] = { w = 102, h = 27 }
SAmotesData[":havlat:"] = { w = 24, h = 27 }
SAmotesData[":haw:"] = { w = 15, h = 15 }
SAmotesData[":hawaaaafap:"] = { w = 38, h = 21 }
SAmotesData[":hb:"] = { w = 27, h = 27 }
SAmotesData[":hchatter:"] = { w = 34, h = 27 }
SAmotesData[":hehe:"] = { w = 15, h = 15 }
SAmotesData[":helladid:"] = { w = 52, h = 27 }
SAmotesData[":hellyeah:"] = { w = 75, h = 27 }
SAmotesData[":henget:"] = { w = 109, h = 30 }
SAmotesData[":heysexy:"] = { w = 17, h = 17 }
SAmotesData[":hf:"] = { w = 10, h = 17 }
SAmotesData[":hfive:"] = { w = 40, h = 17 }
SAmotesData[":hirez:"] = { w = 74, h = 19 }
SAmotesData[":hist101:"] = { w = 34, h = 22 }
SAmotesData[":hitler:"] = { w = 15, h = 15 }
SAmotesData[":holy:"] = { w = 26, h = 20 }
SAmotesData[":holymoley:"] = { w = 102, h = 27 }
SAmotesData[":homebrew:"] = { w = 44, h = 27 }
SAmotesData[":horse:"] = { w = 102, h = 24 }
SAmotesData[":hr:"] = { w = 50, h = 25 }
SAmotesData[":huh:"] = { w = 26, h = 22 }
SAmotesData[":hurr:"] = { w = 99, h = 27 }
SAmotesData[":hydrogen:"] = { w = 27, h = 27 }
SAmotesData[":iamafag:"] = { w = 20, h = 20 }
SAmotesData[":iceburn:"] = { w = 102, h = 27 }
SAmotesData[":iia:"] = { w = 65, h = 22 }
SAmotesData[":iiaca:"] = { w = 65, h = 22 }
SAmotesData[":iiam:"] = { w = 65, h = 22 }
SAmotesData[":iiapa:"] = { w = 65, h = 22 }
SAmotesData[":ins:"] = { w = 8, h = 16 }
SAmotesData[":ironicat:"] = { w = 34, h = 25 }
SAmotesData[":irony:"] = { w = 47, h = 25 }
SAmotesData[":italy:"] = { w = 33, h = 23 }
SAmotesData[":itjb:"] = { w = 25, h = 16 }
SAmotesData[":iw:"] = { w = 92, h = 27 }
SAmotesData[":j:"] = { w = 16, h = 16 }
SAmotesData[":japan:"] = { w = 33, h = 17 }
SAmotesData[":jeb:"] = { w = 19, h = 25 }
SAmotesData[":jerkbag:"] = { w = 27, h = 21 }
SAmotesData[":jewish:"] = { w = 27, h = 19 }
SAmotesData[":jiggled:"] = { w = 52, h = 27 }
SAmotesData[":jihad:"] = { w = 32, h = 22 }
SAmotesData[":joel:"] = { w = 24, h = 24 }
SAmotesData[":jp:"] = { w = 60, h = 27 }
SAmotesData[":justpost:"] = { w = 63, h = 22 }
SAmotesData[":kakashi:"] = { w = 23, h = 23 }
SAmotesData[":kamina:"] = { w = 26, h = 27 }
SAmotesData[":keke:"] = { w = 15, h = 15 }
SAmotesData[":kiddo:"] = { w = 20, h = 20 }
SAmotesData[":killdozer:"] = { w = 63, h = 27 }
SAmotesData[":kimchi:"] = { w = 15, h = 25 }
SAmotesData[":krad:"] = { w = 102, h = 14 }
SAmotesData[":krakentoot:"] = { w = 37, h = 25 }
SAmotesData[":krakken:"] = { w = 55, h = 27 }
SAmotesData[":kratos:"] = { w = 35, h = 19 }
SAmotesData[":laffo:"] = { w = 90, h = 17 }
SAmotesData[":laugh:"] = { w = 35, h = 27 }
SAmotesData[":legion:"] = { w = 92, h = 27 }
SAmotesData[":liara:"] = { w = 92, h = 27 }
SAmotesData[":lol:"] = { w = 32, h = 17 }
SAmotesData[":lost:"] = { w = 47, h = 27 }
SAmotesData[":love:"] = { w = 22, h = 22 }
SAmotesData[":lovewcc:"] = { w = 49, h = 22 }
SAmotesData[":lron:"] = { w = 86, h = 27 }
SAmotesData[":lsd:"] = { w = 32, h = 27 }
SAmotesData[":m10:"] = { w = 102, h = 27 }
SAmotesData[":mad:"] = { w = 15, h = 15 }
SAmotesData[":madmax:"] = { w = 26, h = 25 }
SAmotesData[":magical:"] = { w = 23, h = 20 }
SAmotesData[":manning:"] = { w = 18, h = 22 }
SAmotesData[":mario:"] = { w = 20, h = 26 }
SAmotesData[":mason:"] = { w = 27, h = 27 }
SAmotesData[":master:"] = { w = 102, h = 19 }
SAmotesData[":mcnabb:"] = { w = 17, h = 25 }
SAmotesData[":megaman:"] = { w = 24, h = 26 }
SAmotesData[":mensch:"] = { w = 42, h = 27 }
SAmotesData[":mexico:"] = { w = 46, h = 27 }
SAmotesData[":mil101:"] = { w = 24, h = 20 }
SAmotesData[":milk:"] = { w = 72, h = 27 }
SAmotesData[":milkie:"] = { w = 90, h = 21 }
SAmotesData[":mitt:"] = { w = 66, h = 27 }
SAmotesData[":mmmhmm:"] = { w = 17, h = 17 }
SAmotesData[":mmmsmug:"] = { w = 28, h = 27 }
SAmotesData[":monar:"] = { w = 22, h = 27 }
SAmotesData[":monocle:"] = { w = 23, h = 17 }
SAmotesData[":mordin:"] = { w = 44, h = 25 }
SAmotesData[":moreevil:"] = { w = 82, h = 22 }
SAmotesData[":morning:"] = { w = 31, h = 31 }
SAmotesData[":moustache:"] = { w = 75, h = 22 }
SAmotesData[":mrapig:"] = { w = 23, h = 25 }
SAmotesData[":mrgw:"] = { w = 26, h = 25 }
SAmotesData[":ms:"] = { w = 65, h = 22 }
SAmotesData[":mufasa:"] = { w = 30, h = 25 }
SAmotesData[":mump:"] = { w = 23, h = 25 }
SAmotesData[":munch:"] = { w = 47, h = 27 }
SAmotesData[":nattyburn:"] = { w = 45, h = 27 }
SAmotesData[":neckbeard:"] = { w = 26, h = 24 }
SAmotesData[":negative:"] = { w = 63, h = 49 }
SAmotesData[":newfap:"] = { w = 23, h = 21 }
SAmotesData[":newlol:"] = { w = 18, h = 18 }
SAmotesData[":newt:"] = { w = 29, h = 24 }
SAmotesData[":niggly:"] = { w = 17, h = 17 }
SAmotesData[":nignog:"] = { w = 70, h = 25 }
SAmotesData[":ninja:"] = { w = 17, h = 17 }
SAmotesData[":nixon:"] = { w = 21, h = 27 }
SAmotesData[":nms:"] = { w = 72, h = 18 }
SAmotesData[":nolan:"] = { w = 88, h = 22 }
SAmotesData[":norway:"] = { w = 33, h = 21 }
SAmotesData[":notch:"] = { w = 54, h = 25 }
SAmotesData[":notfunny:"] = { w = 17, h = 25 }
SAmotesData[":nws:"] = { w = 46, h = 16 }
SAmotesData[":nyan:"] = { w = 55, h = 23 }
SAmotesData[":nyd:"] = { w = 36, h = 20 }
SAmotesData[":nyoron:"] = { w = 58, h = 26 }
SAmotesData[":o:"] = { w = 15, h = 15 }
SAmotesData[":obama:"] = { w = 20, h = 20 }
SAmotesData[":objection:"] = { w = 37, h = 21 }
SAmotesData[":ocelot:"] = { w = 30, h = 19 }
SAmotesData[":ohdear:"] = { w = 15, h = 16 }
SAmotesData[":ohdearsass:"] = { w = 20, h = 23 }
SAmotesData[":okpos:"] = { w = 27, h = 27 }
SAmotesData[":orks:"] = { w = 32, h = 18 }
SAmotesData[":ovr:"] = { w = 8, h = 16 }
SAmotesData[":owned:"] = { w = 86, h = 19 }
SAmotesData[":page3:"] = { w = 102, h = 27 }
SAmotesData[":parrot:"] = { w = 37, h = 27 }
SAmotesData[":patriot:"] = { w = 32, h = 28 }
SAmotesData[":patssay:"] = { w = 40, h = 25 }
SAmotesData[":pcgaming1:"] = { w = 102, h = 27 }
SAmotesData[":pcgaming:"] = { w = 40, h = 27 }
SAmotesData[":pedo:"] = { w = 46, h = 19 }
SAmotesData[":pedophiles:"] = { w = 77, h = 27 }
SAmotesData[":pervert:"] = { w = 26, h = 19 }
SAmotesData[":pgi:"] = { w = 27, h = 27 }
SAmotesData[":phone:"] = { w = 47, h = 21 }
SAmotesData[":phoneb:"] = { w = 47, h = 21 }
SAmotesData[":phoneline:"] = { w = 99, h = 19 }
SAmotesData[":pipe:"] = { w = 19, h = 19 }
SAmotesData[":pirate:"] = { w = 23, h = 27 }
SAmotesData[":pluto:"] = { w = 36, h = 27 }
SAmotesData[":poland:"] = { w = 33, h = 18 }
SAmotesData[":pranke:"] = { w = 33, h = 27 }
SAmotesData[":pray:"] = { w = 20, h = 20 }
SAmotesData[":protarget:"] = { w = 99, h = 22 }
SAmotesData[":pseudo:"] = { w = 23, h = 19 }
SAmotesData[":psyberger:"] = { w = 30, h = 25 }
SAmotesData[":psyboom:"] = { w = 62, h = 27 }
SAmotesData[":psyduck:"] = { w = 30, h = 25 }
SAmotesData[":psydwarf:"] = { w = 30, h = 25 }
SAmotesData[":psylon:"] = { w = 30, h = 27 }
SAmotesData[":psypop:"] = { w = 62, h = 27 }
SAmotesData[":pt:"] = { w = 42, h = 22 }
SAmotesData[":punto:"] = { w = 70, h = 27 }
SAmotesData[":pwm:"] = { w = 35, h = 25 }
SAmotesData[":pwn:"] = { w = 26, h = 23 }
SAmotesData[":q:"] = { w = 30, h = 28 }
SAmotesData[":qfg:"] = { w = 21, h = 27 }
SAmotesData[":qirex:"] = { w = 34, h = 27 }
SAmotesData[":qq:"] = { w = 22, h = 22 }
SAmotesData[":qqsay:"] = { w = 88, h = 27 }
SAmotesData[":quagmire:"] = { w = 24, h = 26 }
SAmotesData[":quebec:"] = { w = 33, h = 21 }
SAmotesData[":radcat:"] = { w = 27, h = 25 }
SAmotesData[":raise:"] = { w = 17, h = 17 }
SAmotesData[":ramsay:"] = { w = 72, h = 27 }
SAmotesData[":ranbowdash:"] = { w = 36, h = 25 }
SAmotesData[":rant:"] = { w = 33, h = 22 }
SAmotesData[":razz:"] = { w = 20, h = 20 }
SAmotesData[":reddit:"] = { w = 29, h = 27 }
SAmotesData[":redhammer:"] = { w = 44, h = 25 }
SAmotesData[":regd04:"] = { w = 53, h = 19 }
SAmotesData[":regd05:"] = { w = 62, h = 18 }
SAmotesData[":regd06:"] = { w = 52, h = 17 }
SAmotesData[":regd07:"] = { w = 62, h = 27 }
SAmotesData[":regd08:"] = { w = 102, h = 27 }
SAmotesData[":regd09:"] = { w = 38, h = 25 }
SAmotesData[":regd10:"] = { w = 100, h = 25 }
SAmotesData[":regd11:"] = { w = 74, h = 27 }
SAmotesData[":reject:"] = { w = 15, h = 15 }
SAmotesData[":respek:"] = { w = 18, h = 14 }
SAmotesData[":retrogames:"] = { w = 30, h = 27 }
SAmotesData[":rice:"] = { w = 83, h = 23 }
SAmotesData[":riker:"] = { w = 26, h = 25 }
SAmotesData[":rimshot:"] = { w = 40, h = 27 }
SAmotesData[":riot:"] = { w = 27, h = 25 }
SAmotesData[":rms2:"] = { w = 25, h = 25 }
SAmotesData[":rms:"] = { w = 63, h = 25 }
SAmotesData[":roboluv:"] = { w = 37, h = 17 }
SAmotesData[":rock:"] = { w = 49, h = 18 }
SAmotesData[":rodimus:"] = { w = 46, h = 23 }
SAmotesData[":roflolmao:"] = { w = 82, h = 18 }
SAmotesData[":rolldice:"] = { w = 41, h = 17 }
SAmotesData[":rolleye:"] = { w = 15, h = 15 }
SAmotesData[":rolleyes:"] = { w = 15, h = 15 }
SAmotesData[":rory:"] = { w = 32, h = 32 }
SAmotesData[":rubshands:"] = { w = 47, h = 22 }
SAmotesData[":rudebox:"] = { w = 43, h = 27 }
SAmotesData[":russbus:"] = { w = 102, h = 27 }
SAmotesData[":russo:"] = { w = 42, h = 27 }
SAmotesData[":s:"] = { w = 13, h = 11 }
SAmotesData[":saddowns:"] = { w = 15, h = 15 }
SAmotesData[":sassargh:"] = { w = 36, h = 25 }
SAmotesData[":sax:"] = { w = 29, h = 20 }
SAmotesData[":sbahj:"] = { w = 82, h = 27 }
SAmotesData[":science:"] = { w = 42, h = 27 }
SAmotesData[":scotland:"] = { w = 33, h = 17 }
SAmotesData[":sg:"] = { w = 23, h = 24 }
SAmotesData[":sharpton:"] = { w = 20, h = 25 }
SAmotesData[":shepface:"] = { w = 26, h = 25 }
SAmotesData[":shepicide:"] = { w = 37, h = 27 }
SAmotesData[":shibaz:"] = { w = 27, h = 23 }
SAmotesData[":shibewow:"] = { w = 90, h = 25 }
SAmotesData[":shivdurf:"] = { w = 56, h = 25 }
SAmotesData[":shlick:"] = { w = 52, h = 27 }
SAmotesData[":shobon:"] = { w = 15, h = 15 }
SAmotesData[":shopkeeper:"] = { w = 17, h = 27 }
SAmotesData[":shrek:"] = { w = 25, h = 27 }
SAmotesData[":shroom:"] = { w = 27, h = 27 }
SAmotesData[":shrug:"] = { w = 39, h = 17 }
SAmotesData[":sicknasty:"] = { w = 74, h = 27 }
SAmotesData[":sigh:"] = { w = 17, h = 26 }
SAmotesData[":signings:"] = { w = 32, h = 27 }
SAmotesData[":silent:"] = { w = 15, h = 15 }
SAmotesData[":siren:"] = { w = 22, h = 22 }
SAmotesData[":sissies:"] = { w = 82, h = 23 }
SAmotesData[":slick:"] = { w = 30, h = 27 }
SAmotesData[":smaug:"] = { w = 38, h = 24 }
SAmotesData[":smith:"] = { w = 23, h = 25 }
SAmotesData[":smithcloud:"] = { w = 39, h = 25 }
SAmotesData[":smithfrog:"] = { w = 38, h = 25 }
SAmotesData[":smithicide:"] = { w = 37, h = 27 }
SAmotesData[":smithmouth:"] = { w = 22, h = 27 }
SAmotesData[":smug:"] = { w = 26, h = 25 }
SAmotesData[":smugbert:"] = { w = 20, h = 25 }
SAmotesData[":smugbird:"] = { w = 37, h = 25 }
SAmotesData[":smugdog:"] = { w = 38, h = 24 }
SAmotesData[":smugdroid:"] = { w = 26, h = 25 }
SAmotesData[":smuggo:"] = { w = 26, h = 25 }
SAmotesData[":smugissar:"] = { w = 66, h = 27 }
SAmotesData[":smugndar:"] = { w = 27, h = 26 }
SAmotesData[":smugspike:"] = { w = 26, h = 25 }
SAmotesData[":smugteddie:"] = { w = 26, h = 25 }
SAmotesData[":smugwizard:"] = { w = 45, h = 25 }
SAmotesData[":snoop:"] = { w = 22, h = 27 }
SAmotesData[":solanadumb:"] = { w = 24, h = 24 }
SAmotesData[":sonia:"] = { w = 32, h = 27 }
SAmotesData[":sotw:"] = { w = 84, h = 34 }
SAmotesData[":spain:"] = { w = 33, h = 17 }
SAmotesData[":sparkles:"] = { w = 32, h = 22 }
SAmotesData[":speculate:"] = { w = 92, h = 27 }
SAmotesData[":spergin:"] = { w = 20, h = 25 }
SAmotesData[":spidey:"] = { w = 17, h = 22 }
SAmotesData[":spooky:"] = { w = 28, h = 26 }
SAmotesData[":ssh:"] = { w = 17, h = 20 }
SAmotesData[":ssj:"] = { w = 27, h = 27 }
SAmotesData[":stalker:"] = { w = 98, h = 23 }
SAmotesData[":stare:"] = { w = 18, h = 18 }
SAmotesData[":staredog:"] = { w = 38, h = 24 }
SAmotesData[":stat:"] = { w = 20, h = 25 }
SAmotesData[":steam:"] = { w = 16, h = 16 }
SAmotesData[":stoat:"] = { w = 19, h = 24 }
SAmotesData[":stoke:"] = { w = 92, h = 19 }
SAmotesData[":stonk:"] = { w = 18, h = 18 }
SAmotesData[":sugartits:"] = { w = 26, h = 25 }
SAmotesData[":suicide:"] = { w = 56, h = 27 }
SAmotesData[":sun:"] = { w = 23, h = 23 }
SAmotesData[":supaburn:"] = { w = 48, h = 27 }
SAmotesData[":suspense:"] = { w = 47, h = 27 }
SAmotesData[":sweatdrop:"] = { w = 17, h = 17 }
SAmotesData[":sweden:"] = { w = 33, h = 21 }
SAmotesData[":sweep:"] = { w = 52, h = 52 }
SAmotesData[":swoon:"] = { w = 35, h = 25 }
SAmotesData[":sympathy:"] = { w = 49, h = 22 }
SAmotesData[":synpa:"] = { w = 74, h = 27 }
SAmotesData[":synthy:"] = { w = 28, h = 22 }
SAmotesData[":syoon:"] = { w = 33, h = 27 }
SAmotesData[":taco:"] = { w = 39, h = 25 }
SAmotesData[":tali:"] = { w = 65, h = 22 }
SAmotesData[":tastykake:"] = { w = 39, h = 27 }
SAmotesData[":tbear:"] = { w = 67, h = 17 }
SAmotesData[":techno:"] = { w = 54, h = 27 }
SAmotesData[":tetten:"] = { w = 87, h = 27 }
SAmotesData[":tf:"] = { w = 33, h = 17 }
SAmotesData[":tfrxmas:"] = { w = 50, h = 26 }
SAmotesData[":their:"] = { w = 35, h = 12 }
SAmotesData[":thejoke:"] = { w = 39, h = 25 }
SAmotesData[":thumbsup:"] = { w = 32, h = 27 }
SAmotesData[":thurman:"] = { w = 22, h = 25 }
SAmotesData[":thx:"] = { w = 40, h = 16 }
SAmotesData[":tinfoil:"] = { w = 20, h = 22 }
SAmotesData[":tinsley:"] = { w = 62, h = 17 }
SAmotesData[":tipshat:"] = { w = 27, h = 30 }
SAmotesData[":tito:"] = { w = 33, h = 17 }
SAmotesData[":tizzy:"] = { w = 19, h = 19 }
SAmotesData[":todd:"] = { w = 52, h = 27 }
SAmotesData[":toot:"] = { w = 24, h = 25 }
SAmotesData[":tootzzz:"] = { w = 31, h = 25 }
SAmotesData[":toughguy:"] = { w = 28, h = 24 }
SAmotesData[":toxx:"] = { w = 102, h = 27 }
SAmotesData[":troll:"] = { w = 25, h = 25 }
SAmotesData[":tubular:"] = { w = 82, h = 27 }
SAmotesData[":turianass:"] = { w = 39, h = 27 }
SAmotesData[":tviv:"] = { w = 102, h = 22 }
SAmotesData[":tvtropes:"] = { w = 45, h = 25 }
SAmotesData[":twentyfour:"] = { w = 36, h = 27 }
SAmotesData[":twisted:"] = { w = 15, h = 15 }
SAmotesData[":ughh:"] = { w = 70, h = 47 }
SAmotesData[":uhaul:"] = { w = 97, h = 27 }
SAmotesData[":unsmigghh:"] = { w = 34, h = 27 }
SAmotesData[":unsmith:"] = { w = 23, h = 25 }
SAmotesData[":ussr:"] = { w = 33, h = 17 }
SAmotesData[":v:"] = { w = 15, h = 15 }
SAmotesData[":vd:"] = { w = 56, h = 27 }
SAmotesData[":vick:"] = { w = 102, h = 27 }
SAmotesData[":viconia:"] = { w = 25, h = 25 }
SAmotesData[":viggo:"] = { w = 46, h = 25 }
SAmotesData[":vince:"] = { w = 46, h = 25 }
SAmotesData[":w-hat:"] = { w = 30, h = 15 }
SAmotesData[":w00t:"] = { w = 86, h = 19 }
SAmotesData[":w2byob:"] = { w = 37, h = 22 }
SAmotesData[":wal:"] = { w = 32, h = 25 }
SAmotesData[":waycool:"] = { w = 61, h = 20 }
SAmotesData[":wcc:"] = { w = 19, h = 19 }
SAmotesData[":wcw:"] = { w = 33, h = 17 }
SAmotesData[":weed:"] = { w = 60, h = 25 }
SAmotesData[":what:"] = { w = 17, h = 17 }
SAmotesData[":whatup:"] = { w = 102, h = 27 }
SAmotesData[":whip:"] = { w = 31, h = 22 }
SAmotesData[":whoptc:"] = { w = 58, h = 25 }
SAmotesData[":wiggle:"] = { w = 17, h = 17 }
SAmotesData[":wink:"] = { w = 92, h = 27 }
SAmotesData[":witch:"] = { w = 21, h = 22 }
SAmotesData[":wmwink:"] = { w = 90, h = 25 }
SAmotesData[":wom:"] = { w = 102, h = 27 }
SAmotesData[":woof:"] = { w = 54, h = 27 }
SAmotesData[":wookie:"] = { w = 30, h = 27 }
SAmotesData[":woop:"] = { w = 27, h = 24 }
SAmotesData[":wooper:"] = { w = 32, h = 26 }
SAmotesData[":words:"] = { w = 48, h = 17 }
SAmotesData[":worship:"] = { w = 28, h = 26 }
SAmotesData[":wotwot:"] = { w = 27, h = 27 }
SAmotesData[":wow:"] = { w = 65, h = 27 }
SAmotesData[":woz:"] = { w = 33, h = 17 }
SAmotesData[":wrongful:"] = { w = 87, h = 27 }
SAmotesData[":wtc:"] = { w = 58, h = 25 }
SAmotesData[":wtf:"] = { w = 37, h = 17 }
SAmotesData[":wth:"] = { w = 21, h = 18 }
SAmotesData[":xcom:"] = { w = 92, h = 27 }
SAmotesData[":xd:"] = { w = 15, h = 15 }
SAmotesData[":xie:"] = { w = 52, h = 27 }
SAmotesData[":yarr:"] = { w = 22, h = 20 }
SAmotesData[":yayclod:"] = { w = 39, h = 25 }
SAmotesData[":yaycloud:"] = { w = 39, h = 25 }
SAmotesData[":yohoho:"] = { w = 88, h = 27 }
SAmotesData[":yosbutt:"] = { w = 38, h = 27 }
SAmotesData[":yoshi:"] = { w = 17, h = 27 }
SAmotesData[":yotj:"] = { w = 27, h = 25 }
SAmotesData[":yousuck:"] = { w = 75, h = 14 }
SAmotesData[":yum:"] = { w = 31, h = 18 }
SAmotesData[":zaeed:"] = { w = 92, h = 27 }
SAmotesData[":zerg:"] = { w = 62, h = 17 }
SAmotesData[":zoid:"] = { w = 26, h = 27 }
SAmotesData[":zombie:"] = { w = 15, h = 15 }
SAmotesData[":zoro:"] = { w = 25, h = 25 }
SAmotesData[":zpatriot:"] = { w = 32, h = 25 }
SAmotesData[":;):"] = { w = 15, h = 15 }
SAmotesData[":;-*:"] = { w = 15, h = 15 }

local kstrColorChatImage = "white"

-- Copied Chatlog constants

local kstrColorChatRegular 	= "ff7fffb9"
local kstrColorChatShout	= "ffd9eef7"
local kstrColorChatRoleplay = "ff58e3b0"
local kstrBubbleFont 		= "CRB_Dialog"
local kstrDialogFont 		= "CRB_Dialog"
local kstrDialogFontRP 		= "CRB_Dialog_I"

local kstrGMIcon 		= "Icon_Windows_UI_GMIcon"

local karEvalColors =
{
	[Item.CodeEnumItemQuality.Inferior] 		= "ItemQuality_Inferior",
	[Item.CodeEnumItemQuality.Average] 			= "ItemQuality_Average",
	[Item.CodeEnumItemQuality.Good] 			= "ItemQuality_Good",
	[Item.CodeEnumItemQuality.Excellent] 		= "ItemQuality_Excellent",
	[Item.CodeEnumItemQuality.Superb] 			= "ItemQuality_Superb",
	[Item.CodeEnumItemQuality.Legendary] 		= "ItemQuality_Legendary",
	[Item.CodeEnumItemQuality.Artifact]		 	= "ItemQuality_Artifact",
}

local karChannelTypeToColor = 
{
	[ChatSystemLib.ChatChannel_Command] 		= { Channel = "ChannelCommand", 		},
	[ChatSystemLib.ChatChannel_System] 			= { Channel = "ChannelSystem", 			},
	[ChatSystemLib.ChatChannel_Debug] 			= { Channel = "ChannelDebug", 			},
	[ChatSystemLib.ChatChannel_Say] 			= { Channel = "ChannelSay", 			},
	[ChatSystemLib.ChatChannel_Yell] 			= { Channel = "ChannelShout", 			},
	[ChatSystemLib.ChatChannel_Whisper] 		= { Channel = "ChannelWhisper", 		},
	[ChatSystemLib.ChatChannel_Party] 			= { Channel = "ChannelParty", 			},
	[ChatSystemLib.ChatChannel_Emote] 			= { Channel = "ChannelEmote", 			},
	[ChatSystemLib.ChatChannel_AnimatedEmote] 	= { Channel = "ChannelEmote", 			},
	[ChatSystemLib.ChatChannel_Zone] 			= { Channel = "ChannelZone", 			},
	[ChatSystemLib.ChatChannel_ZonePvP] 		= { Channel = "ChannelPvP", 			},
	[ChatSystemLib.ChatChannel_Trade] 			= { Channel = "ChannelTrade",			},
	[ChatSystemLib.ChatChannel_Guild] 			= { Channel = "ChannelGuild", 			},
	[ChatSystemLib.ChatChannel_GuildOfficer] 	= { Channel = "ChannelGuildOfficer",	},
	[ChatSystemLib.ChatChannel_Society] 		= { Channel = "ChannelCircle2",			},
	[ChatSystemLib.ChatChannel_Custom] 			= { Channel = "ChannelCustom", 			},
	[ChatSystemLib.ChatChannel_NPCSay] 			= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_NPCYell] 		= { Channel = "ChannelNPC",		 		},
	[ChatSystemLib.ChatChannel_NPCWhisper]		= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_Datachron] 		= { Channel = "ChannelNPC", 			},
	[ChatSystemLib.ChatChannel_Combat] 			= { Channel = "ChannelGeneral", 		},
	[ChatSystemLib.ChatChannel_Realm] 			= { Channel = "ChannelSupport", 		},
	[ChatSystemLib.ChatChannel_Loot] 			= { Channel = "ChannelLoot", 			},
	[ChatSystemLib.ChatChannel_PlayerPath] 		= { Channel = "ChannelGeneral", 		},
	[ChatSystemLib.ChatChannel_Instance] 		= { Channel = "ChannelParty", 			},
	[ChatSystemLib.ChatChannel_WarParty] 		= { Channel = "ChannelWarParty",		},
	[ChatSystemLib.ChatChannel_WarPartyOfficer] = { Channel = "ChannelWarPartyOfficer", },
	[ChatSystemLib.ChatChannel_Advice] 			= { Channel = "ChannelAdvice", 			},
	[ChatSystemLib.ChatChannel_AccountWhisper] 	= { Channel = "ChannelAccountWisper", 	},
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function SAmotes:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function SAmotes:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- SAmotes OnLoad
-----------------------------------------------------------------------------------------------
function SAmotes:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("SAmotes.xml")
	Apollo.LoadSprites("SAmotesSprites.xml", "SAmotesSprites")
	Apollo.GetPackage("Gemini:Hook-1.0").tPackage:Embed(self)
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- SAmotes OnDocLoaded
-----------------------------------------------------------------------------------------------
function SAmotes:OnDocLoaded()
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		--[[
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "SAmotesForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		self:InitialiseWindow()
	    self.wndMain:Show(false, true)
		--]]
		Apollo.RegisterSlashCommand("samotes", "OnSAmotesOn", self)
		self:RawHook(Apollo.GetAddon("ChatLog"),"HelperGenerateChatMessage")
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("SAmotesInterfaceMenuEvent", "OnSAmotesOn", self)
	end
end

function SAmotes:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "SAmotes", {"SAmotesInterfaceMenuEvent", "" , "SAmotesSprites:smugicon" })
end

-----------------------------------------------------------------------------------------------
-- Hooked Function
-----------------------------------------------------------------------------------------------
-- full replacement of Carbine's chatlog message generator
-- most of this is straight copy from the chatlog addon
-- additions are marked with comments so its (hopefully) easy to update if they change shit
-- also various references to 'self' are replaced with 'luaChatLog'

function SAmotes:HelperGenerateChatMessage(luaChatLog,tQueuedMessage)
	if tQueuedMessage.xml then
		return
	end

	local eChannelType = tQueuedMessage.eChannelType
	local tMessage = tQueuedMessage.tMessage

	-- Different handling for combat log
	if eChannelType == ChatSystemLib.ChatChannel_Combat then
		-- no formats in combat, roll it all up into one.
		local strMessage = ""
		for idx, tSegment in ipairs(tMessage.arMessageSegments) do
			strMessage = strMessage .. tSegment.strText
		end
		tQueuedMessage.strMessage = strMessage
		return
	end

	-- SAMOTES ADDITION
	if not (tMessage and tMessage.arMessageSegments and eChannelType) then
		return
	end
	tMessage.arMessageSegments = self:ProcessMessageSegments(tMessage.arMessageSegments)
	-- END OF ADDITION

	local xml = XmlDoc.new()
	local tm = GameLib.GetLocalTime()
	local crText = luaChatLog.arChatColor[eChannelType] or ApolloColor.new("white")
	local crChannel = ApolloColor.new(karChannelTypeToColor[eChannelType].Channel or "white")
	local crPlayerName = ApolloColor.new("ChatPlayerName")

	
	local strTime = "" if luaChatLog.bShowTimestamp then strTime = string.format("%d:%02d ", tm.nHour, tm.nMinute) end
	local strWhisperName = tMessage.strSender
	if tMessage.strRealmName:len() > 0 then
		-- Name/Realm formatting needs to be very specific for cross realm chat to work
		strWhisperName = strWhisperName .. "@" .. tMessage.strRealmName
	end

	--strWhisperName must only be sender@realm, or friends equivelent name.

	local strPresenceState = ""
	if tMessage.bAutoResponse then
		strPresenceState = '('..Apollo.GetString("AutoResponse_Prefix")..')'
	end

	if tMessage.nPresenceState == FriendshipLib.AccountPresenceState_Away then
		strPresenceState = '<'..Apollo.GetString("Command_Friendship_AwayFromKeyboard")..'>'
	elseif tMessage.nPresenceState == FriendshipLib.AccountPresenceState_Busy then
		strPresenceState = '<'..Apollo.GetString("Command_Friendship_DoNotDisturb")..'>'
	end

	if eChannelType == ChatSystemLib.ChatChannel_Whisper then
		if not tMessage.bSelf then
			luaChatLog.tLastWhisperer = { strCharacterName = strWhisperName, eChannelType = ChatSystemLib.ChatChannel_Whisper }--record the last incoming whisperer for quick response
		end
		Sound.Play(Sound.PlayUISocialWhisper)
	elseif eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then

		local tPreviousWhisperer = luaChatLog.tLastWhisperer

		luaChatLog.tLastWhisperer =
		{
			strCharacterName = tMessage.strSender,
			strRealmName = nil,
			strDisplayName = nil,
			eChannelType = ChatSystemLib.ChatChannel_AccountWhisper
		}

		local tAccountFriends = FriendshipLib.GetAccountList()
		for idx, tAccountFriend in pairs(tAccountFriends) do
			if tAccountFriend.arCharacters ~= nil then
				for idx, tCharacter in pairs(tAccountFriend.arCharacters) do
					if tCharacter.strCharacterName == tMessage.strSender and (tMessage.strRealmName:len() == 0 or tCharacter.strRealm == tMessage.strRealmName) then
						if not tMessage.bSelf or (tPreviousWhisperer and tPreviousWhisperer.strCharacterName == tMessage.strSender) then
							luaChatLog.tLastWhisperer.strDisplayName = tAccountFriend.strCharacterName
							luaChatLog.tLastWhisperer.strRealmName = tCharacter.strRealm
						end
						strWhisperName = tAccountFriend.strCharacterName
						if tMessage.strRealmName:len() > 0 then
							-- Name/Realm formatting needs to be very specific for cross realm chat to work
							strWhisperName = strWhisperName .. "@" .. tMessage.strRealmName
						end
					end
				end
			end
		end
		Sound.Play(Sound.PlayUISocialWhisper)
	end

	-- We build strings backwards, right to left
	if eChannelType == ChatSystemLib.ChatChannel_AnimatedEmote then -- emote animated channel gets special formatting
		xml:AddLine(strTime, crChannel, luaChatLog.strFontOption, "Left")

	elseif eChannelType == ChatSystemLib.ChatChannel_Emote then -- emote channel gets special formatting
		xml:AddLine(strTime, crChannel, luaChatLog.strFontOption, "Left")
		if strWhisperName:len() > 0 then
			if tMessage.bGM then
				xml:AppendImage(kstrGMIcon, 16, 16)
			end
			xml:AppendText(strWhisperName, crPlayerName, luaChatLog.strFontOption, {CharacterName=strWhisperName, nReportId=tMessage.nReportId}, "Source")
		end
		xml:AppendText(" ")
	else
		local strChannel
		if eChannelType == ChatSystemLib.ChatChannel_Society then
			strChannel = String_GetWeaselString(Apollo.GetString("ChatLog_GuildCommand"), tQueuedMessage.strChannelName, tQueuedMessage.strChannelCommand)
		else
			strChannel = String_GetWeaselString(Apollo.GetString("CRB_Brackets_Space"), tQueuedMessage.strChannelName)
		end

		if luaChatLog.bShowChannel ~= true then
			strChannel = ""
		end

		xml:AddLine(strTime .. strChannel, crChannel, luaChatLog.strFontOption, "Left")
		if strWhisperName:len() > 0 then

			local strWhisperNamePrefix = ""
			if eChannelType == ChatSystemLib.ChatChannel_Whisper or eChannelType == ChatSystemLib.ChatChannel_AccountWhisper then
				if tMessage.bSelf then
					strWhisperNamePrefix = Apollo.GetString("ChatLog_To")
				else
					strWhisperNamePrefix = Apollo.GetString("ChatLog_From")
				end
			end

			xml:AppendText( strWhisperNamePrefix, crText, luaChatLog.strFontOption)

			if tMessage.bGM then
				xml:AppendImage(kstrGMIcon, 16, 16)
			end

			xml:AppendText( strWhisperName, crPlayerName, luaChatLog.strFontOption, {CharacterName=strWhisperName, nReportId=tMessage.nReportId}, "Source")
		end
		xml:AppendText( strPresenceState .. ": ", crChannel, luaChatLog.strFontOption, "Left")
	end

	local xmlBubble = nil
	if tMessage.bShowChatBubble then
		xmlBubble = XmlDoc.new() -- This is the speech bubble form
		xmlBubble:AddLine("", crChannel, luaChatLog.strFontOption, "Center")
	end

	local bHasVisibleText = false
	for idx, tSegment in ipairs( tMessage.arMessageSegments ) do
		local strText = tSegment.strText
		local bAlien = tSegment.bAlien or tMessage.bCrossFaction
		local bShow = false

		if luaChatLog.eRoleplayOption == 3 then
			bShow = not tSegment.bRolePlay
		elseif luaChatLog.eRoleplayOption == 2 then
			bShow = tSegment.bRolePlay
		else
			bShow = true;
		end

		if bShow then
			local crChatText = crText;
			local crBubbleText = kstrColorChatRegular
			local strChatFont = luaChatLog.strFontOption
			local strBubbleFont = kstrBubbleFont
			local tLink = {}


			if tSegment.uItem ~= nil then -- item link
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uItem:GetName())
				crChatText = karEvalColors[tSegment.uItem:GetItemQuality()]
				crBubbleText = ApolloColor.new("white")

				tLink.strText = strText
				tLink.uItem = tSegment.uItem

			elseif tSegment.uQuest ~= nil then -- quest link
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uQuest:GetTitle())
				crChatText = ApolloColor.new("green")
				crBubbleText = ApolloColor.new("green")

				tLink.strText = strText
				tLink.uQuest = tSegment.uQuest

			elseif tSegment.uArchiveArticle ~= nil then -- archive article
				-- replace me with correct colors
				strText = String_GetWeaselString(Apollo.GetString("CRB_Brackets"), tSegment.uArchiveArticle:GetTitle())
				crChatText = ApolloColor.new("ffb7a767")
				crBubbleText = ApolloColor.new("ffb7a767")

				tLink.strText = strText
				tLink.uArchiveArticle = tSegment.uArchiveArticle

			-- SAMOTES ADDITION
			elseif tSegment.strImage ~= nil then
				strText = ""
				if idx == #tMessage.arMessageSegments then -- fix for images occasionally disappearing when they're the final segment
					strText = " "
				end
				if SAmotesData[tSegment.strImage] then
					xml:AppendImage("SAmotesSprites:"..string.gsub(tSegment.strImage,":",""),SAmotesData[tSegment.strImage].w,SAmotesData[tSegment.strImage].h)
				end			
			-- END OF ADDITION
			
			else
				if tSegment.bRolePlay then
					crBubbleText = kstrColorChatRoleplay
					strChatFont = luaChatLog.strRPFontOption
					strBubbleFont = kstrDialogFontRP
				end

				if bAlien or tSegment.bProfanity then -- Weak filter. Note only profanity is scrambled.
					strChatFont = luaChatLog.strAlienFontOption
					strBubbleFont = luaChatLog.strAlienFontOption
				end
			end

			if next(tLink) == nil then
				xml:AppendText(strText, crChatText, strChatFont)
			else
				local strLinkIndex = tostring( luaChatLog:HelperSaveLink(tLink) )
				-- append text can only save strings as attributes.
				xml:AppendText(strText, crChatText, strChatFont, {strIndex=strLinkIndex} , "Link")
			end

			if xmlBubble then
				-- SAMOTES ADDITION
				if tSegment.strImage ~= nil then
					if SAmotesData[tSegment.strImage] then
						xmlBubble:AppendImage("SAmotesSprites:"..string.gsub(tSegment.strImage,":",""),SAmotesData[tSegment.strImage].w,SAmotesData[tSegment.strImage].h)
					end
				end			
				-- END OF ADDITION
				xmlBubble:AppendText(strText, crBubbleText, strBubbleFont) -- Format for bubble; regular
			end

			bHasVisibleText = bHasVisibleText or luaChatLog:HelperCheckForEmptyString(strText)
		end
	end

	tQueuedMessage.bHasVisibleText = bHasVisibleText
	tQueuedMessage.xml = xml
	tQueuedMessage.xmlBubble = xmlBubble
end


-----------------------------------------------------------------------------------------------
-- SAmotes Functions
-----------------------------------------------------------------------------------------------

function SAmotes:ProcessMessageSegments(arMessageSegments)
	local arNewSegments = {}
	for idx, tSegment in ipairs(arMessageSegments) do
		if tSegment.strText then
			local tStringChunks = self:ProcessString(tSegment.strText)
			for _,tChunk in ipairs(tStringChunks) do
				local nId = #arNewSegments+1
				arNewSegments[nId] = {}
				for k,v in pairs(tSegment) do
					arNewSegments[nId][k] = v
				end
				if tChunk.type == "image" then
					arNewSegments[nId].strText = nil
					arNewSegments[nId].strImage = tChunk.data
				else
					arNewSegments[nId].strText = tChunk.data
				end
			end
		end
	end
	return arNewSegments
end

function SAmotes:ProcessString(str)
	local tStringChunks = {}
	
	-- Special case emotes like :) or :( are bracketed with colons before processing
	for k,v in pairs(SAmotesSpecial) do
		if v then
			str = string.gsub(str,k,v)
		end
	end
	
	-- Get all colon positions in original string
	local tColonPositions = {}
	local nPos = 1
	local nCount = 1
	while string.find(str,":",nPos) do
		tColonPositions[nCount] = string.find(str,":",nPos)
		nPos = string.find(str,":",nPos) + 1
		nCount = nCount + 1
	end
	
	-- Check all text between colons to see if its a valid emote
	-- if so, then add chunks to the table
	nPos = 1
	nCount = 1
	for k in ipairs(tColonPositions) do
		if tColonPositions[k] >= nPos and tColonPositions[k+1] then
			local strEmote = string.sub(str,tColonPositions[k],tColonPositions[k+1])
			if SAmotesData[strEmote] then
				if string.len(string.sub(str,nPos,tColonPositions[k]-1))>0 then
					tStringChunks[nCount] = {}
					tStringChunks[nCount].data = string.sub(str,nPos,tColonPositions[k]-1)
					tStringChunks[nCount].type = "string"
					nCount = nCount+1
				end
				tStringChunks[nCount] = {}
				tStringChunks[nCount].data = strEmote
				tStringChunks[nCount].type = "image"
				nCount = nCount+1
				nPos = tColonPositions[k+1]+1
			end
		end
	end
	
	-- Add remainder of string as final chunk
	if string.len(string.sub(str,nPos))>0 then
		tStringChunks[nCount] = {}
		tStringChunks[nCount].data = string.sub(str,nPos)
		tStringChunks[nCount].type = "string"
	end
	return tStringChunks

end

-----------------------------------------------------------------------------------------------
-- SAmotesForm Functions
-----------------------------------------------------------------------------------------------

function SAmotes:InitialiseWindow()
	if self.wndMain then
		return
	end
	self.wndMain = Apollo.LoadForm(self.xmlDoc, "SAmotesForm", nil, self)
	if self.wndMain == nil then
		Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
		return
	end
	self.wndMain:Show(true)
	local wndList = self.wndMain:FindChild("ListWindow")
	if not wndList then
		return
	end
	local tSort = {}
	for k in pairs(SAmotesData) do
		table.insert(tSort,k)	
	end
	table.sort(tSort)
	for _,k in ipairs(tSort) do
		v = SAmotesData[k]
		local strText = k
		for orig,replace in pairs(SAmotesSpecial) do
			if k == replace then
				strText = string.gsub(orig,"%%","")
				break
			end
		end
		local wndListEntry = Apollo.LoadForm(self.xmlDoc,"ListEntry",wndList,self)
		wndListEntry:FindChild("Name"):SetText(strText)
		wndListEntry:FindChild("Image"):SetSprite("SAmotesSprites:"..string.gsub(k,":",""))
		wndListEntry:FindChild("Image"):SetAnchorOffsets(0,0,v.w,v.h)
		wndListEntry:SetAnchorOffsets(0,0,0,math.max(v.h,30))
	end
	wndList:ArrangeChildrenVert()
end

function SAmotes:CloseWindow()
	if self.wndMain then
		self.wndMain:Destroy()
		self.wndMain = nil
	end
end

function SAmotes:OnListEntryClick( wndHandler, wndControl, eMouseButton, nLastRelativeMouseX, nLastRelativeMouseY )
	Apollo.GetAddon("ChatLog"):HelperGetCurrentEditbox():InsertText(wndControl:GetParent():FindChild("Name"):GetText())
end

function SAmotes:OnSAmotesOn()
	if self.wndMain then
		self:CloseWindow()
	else
		self:InitialiseWindow()
	end
end

-- when the Cancel button is clicked
function SAmotes:OnCancel()
	self:CloseWindow()
end

-----------------------------------------------------------------------------------------------
-- SAmotes Instance
-----------------------------------------------------------------------------------------------
local SAmotesInst = SAmotes:new()
SAmotesInst:Init()
